/* See dm_cand_list.h */
/* Last edited on 2006-12-22 22:33:39 by stolfi */

#define dm_cand_list_C_COPYRIGHT \
  "Copyright  2006  by the State University of Campinas (UNICAMP)"

#include <dm_basic.h>
#include <dm_seq.h>
#include <dm_pairing.h>
#include <dm_cand.h>
#include <dm_cand_list.h>

#include <affirm.h>
#include <fget.h>
#include <filefmt.h>

#include <assert.h>
#include <stdio.h>
#include <math.h>

void dm_cand_list_insert
  ( dm_cand_vec_t *cdv,
    int *ncandP,
    int maxCands,
    dm_cand_t *cd,
    double frac
  )
  {
    auto int bubble_up(int j);
      /* Assumes that the list is OK except that {cdv[j]} may be worse
        than {cdv[j-1]}. Swaps the two. */
    
    auto void delete(int j);
      /* Deletes the entry {cdv[j]}, displacing all subsequent entries. 
        Decrements {ncand}. */
    
    int k; /* Final position of inserted candidate, or {(*ncandP)} if not inserted. */

    /* Find the smallest {k} such that {cdv[k]} is definitely worse than {cd}: */
    k = (*ncandP);
    while ((k > 0) && (cdv->el[k-1].score < cd->score)) { k--; }

    if (frac >= 0.0)
      { /* Discard {cd} if there is any better or equal cand that contains or is similar to {cd}: */
        int j;
        for (j = 0; j < k; j++)
          { int ngnew = dm_pairing_num_rungs(cd->pr); /* Rungs in {cd}. */
            int ngboth = dm_cand_count_common_rungs(&(cdv->el[j]), cd); /* Shared rungs. */
            if (ngboth >= frac*ngnew) 
              { /* Discard {cd}: */
                dm_cand_debug("C", cd);
                return;
              }
          }

        /* Remove any candidates that are worse than {cd} but contained in or similar to {cd}: */
        for (j = (*ncandP)-1; j >= k; j--)
          { int ngold = dm_pairing_num_rungs(cdv->el[j].pr); /* Rungs in {cdv->el[j]}. */
            int ngboth = dm_cand_count_common_rungs(&(cdv->el[j]), cd); /* Shared rungs. */
            if (ngboth >= frac*ngold) 
              { /* Discard candidate {cvd.el[j]}: */
                dm_cand_debug("S", &(cdv->el[j]));
                delete(j);
              }
          }
      }

    if ((*ncandP) >= maxCands)
      { k = (*ncandP)-1;
        if (cd->score < cdv->el[k].score)
          { /* Discard {cd}: */
            dm_cand_debug("B", cd);
            return;
          }
        else
          { /* Delete worst candidate: */
            dm_cand_debug("-", &(cdv->el[k]));
            delete(k);
          }
      }

    /* Add {cd} to {cdv[0..ncand-1]}: */
    assert((*ncandP) < maxCands);
    k = (*ncandP);
    dm_cand_vec_expand(cdv, k);
    cdv->el[k] = (*cd);
    (*ncandP)++;
    k = bubble_up(k);
    assert((k >= 0) && (k < (*ncandP)));
    dm_cand_debug("+", &(cdv->el[k]));

    int bubble_up(int j)
      { while (j > 0)
          { /* Get parent: */
            int k = j-1; 
            /* If {cdv[k]} is not worse than {cdv[j]}, stop: */
            if (cdv->el[k].score >= cdv->el[j].score) { break; }
            /* Swap parent with {cdv[j]}: */
            dm_cand_t pp = cdv->el[k]; cdv->el[k] = cdv->el[j]; cdv->el[j] = pp;
            j = k;
          }
        return j;
      }

    void delete(int j)
      { int i;
        for (i = j+1; i < (*ncandP); i++) { cdv->el[i-1] = cdv->el[i]; }
        (*ncandP)--;
      }
  }
