#ifndef dm_image_H
#define dm_image_H

/* Image tools for {dm_match}. */
/* Last edited on 2006-12-25 20:15:24 by stolfi */

#define dm_image_H_COPYRIGHT \
  "Copyright  2005  by the State University of Campinas (UNICAMP)"

#include <dm_basic.h>
#include <stdint.h>
#include <float_image.h>

typedef float_image_t dm_image_t; 
   /* An image, for similarity maps and things like that. */

void dm_alloc_image(int nx, int ny, dm_image_t **imgp, int *scalep);
  /* Allocates a grayscale image (single-channel {dm_image_t}) to plot
    a matrix of {nx} columns and {ny} rows. The image has size
    {ceil(nx/scale)} by {ceil(ny/scale)}, where {scale} is chosen so
    that neither dimension exceeds some reasonable maximum. Returns the
    image pointer in {*imgp} and the chosen scale in {*scalep}. */

void dm_bump_image(dm_image_t *img, int ix, int iy, int scale, float v);
  /* Add {v} to the pixel of {img} corresponding to index pair {(ix,iy)}
    assuming that the image has the specified {scale} factor.
    Namely, adds {v} to pixel in column {ix/scale} and row {iy/scale}. 
    If that pixel is {-oo}, sets it to {v}. */

/* CHOOSING OUTPUT PIXEL RANGES

  The procedures in this section choose the low value {lo} and high value {hi} 
  that should be mapped to black and white (in either order) for better 
  visualization of a given image {img}.  In any case, the limits {lo} and {hi}
  are fudged as needed so as to ensure that {lo < hi}. */
  
void dm_compute_avg_dev_image_range(dm_image_t *img, double *lo, double *hi, double ns);
  /* Computes the average {avg} and standard deviation {dev} of all
    *finite* pixel values. The {lo} and {hi} values are set to 
    {avgns*dev}, respectively. The deviation is fudged up as needed to
    make those two values distinct. */ 

void dm_compute_min_max_image_range(dm_image_t *img, double *lo, double *hi);
  /* Computes the minimum {vmin} and maximum {vmax} of all *finite*
    pixel values. The {lo} and {hi} values are set to {vmin} and {vmax},
    respectively. */ 

float_image_t *dm_image_colorize_float_image(dm_image_t *gim, double maxf);
  /* Converts a single-channel image {gim} to a pseudocolor image {cim}.
    A value {v} of {gim} is mapped to blue if equal to {-maxf} or less,
    to red if  equal to {+maxf} or greater, and to white if close to zero.
    If {v} is {INF}, it is treated as `undefined' and mapped to black. */

void dm_write_float_image_as_pnm
  ( dm_image_t *img, 
    double minv, 
    double maxv, 
    char *name,
    char *tag
  );
  /* Writes the image {img} to disk as a PGM or PPM image, with
    file name "{name}{tag}.pgm", maxval 255, converting affinely the
    samples from {[minv _ maxv]} to {[0..maxval]} with clipping.
    The output is PGM if {img} has 1 channel, and PPM if it has
    3 channels. */ 

#endif
