/* See dm_multi.h */
/* Last edited on 2007-01-04 01:21:29 by stolfi */ 

#define dm_multi_C_COPYRIGHT \
  "Copyright  2005  by the State University of Campinas (UNICAMP)" \
  " and the Federal Fluminense University (UFF)"

#include <dm_basic.h>
#include <dm_seq.h>
#include <dm_cand.h>
#include <dm_cand_list.h>
#include <dm_multi.h>
#include <dm_refine.h>

#include <assert.h>

void dm_multi_seq_filter
  ( dm_seq_t *s, 
    int maxLevel, 
    double_vec_t *wtb0,
    char *wname0,
    double_vec_t *wtb1,
    char *wname1,
    dm_seq_t sr[]
  )
  { sr[0] = *s;
    int level;
    for (level = 1; level <= maxLevel; level++)
      { double_vec_t *wtb = (level == 1 ? wtb0 : wtb1);
        char *wname = (level == 1 ? wname0 : wname1);
        sr[level] = dm_seq_filter(&(sr[level-1]), wtb, wname);
      }
  }

void dm_multi_seq_free(dm_seq_t sr[], int maxLevel)
  { int level;
    for (level = 1; level <= maxLevel; level++)
      { free(sr[level].dv.el); }
  }
   
void dm_multi_rung_vec_filter
  ( dm_rung_vec_t *gv,
    int nx,
    bool_t circx,
    int ny,
    bool_t circy,
    int maxLevel, 
    int nw0, 
    int nw1, 
    dm_rung_vec_t gvr[]
  )
  { gvr[0] = *gv;
    int nxold = nx, nyold = ny;
    int level;
    for (level = 1; level <= maxLevel; level++)
      { int nw = (level == 1 ? nw0 : nw1);
        int nxnew = (nxold - nw + 1)/2, nynew = (nyold - nw + 1)/2;
        gvr[level] = dm_rung_vec_map_to_coarser
          (&(gvr[level-1]), nxold, nyold, nxnew, nynew, nw);
        nxold = nxnew; nyold = nynew;
      }
  }

void dm_multi_rung_vec_free(dm_rung_vec_t gvr[], int maxLevel)
  { int level;
    for (level = 1; level <= maxLevel; level++)
      { free(gvr[level].el); }
  }

dm_cand_vec_t dm_multi_find_matches
  ( dm_cand_vec_t *cdvini,
    dm_seq_t a[], 
    dm_seq_t b[],
    int maxMatches,
    int maxLevel,
    int minBases,
    int nwtb0,
    int nwtb1,
    double frac,
    bool_t refine,
    int delta,
    int kappa,
    bool_t shrink,
    dm_step_score_proc_t step_score,
    char *name
  )
  { /* Working matrices: */
    dm_dyn_tableau_t tb = dm_dyn_tableau_new(); 

    /* Current candidate set: */
    dm_cand_vec_t cdv;
    
    /* Multiscale search and refine at  scales: */
    int level;
    for (level = maxLevel; level >= 0; level--)
      { fprintf(stderr, "\n");
        fprintf(stderr, "= = = = = = = = = = = = = = = = = = = = = = = = = = = = = =\n");
        fprintf(stderr, "LEVEL %d\n", level);
        
        /* Filename prefix for current level: */
        char *levName = NULL;
        if (name != NULL) { asprintf(&levName, "%s-%02d", name, level); }
        
        /* Scale reduction factor from original strings: */
        int scale = 1 << level;
        
        /* Strings that are being matched at this level: */
        dm_seq_t *ap = &(a[level]);
        dm_seq_t *bp = &(b[level]);
        
        /* Their lengths and circularities: */
        int na = dm_seq_length(ap); bool_t circa = dm_seq_is_circular(ap);
        int nb = dm_seq_length(bp); bool_t circb = dm_seq_is_circular(bp);
        
        if (levName != NULL)
          { /* Plot pair-to-pair distances: */;
            dm_pairing_seq_seq_image_write_named(ap, bp, step_score, levName, "-eq-cur");
          }
            
        if (level == maxLevel)
          { cdv = (*cdvini); }
        else
          { 
            fprintf(stderr, "mapping %d candidates", cdv.nel);
            fprintf(stderr, " from level %d to level %d ...\n", level+1, level);
            /* Map the candidates: */
            int nwtb = (level == 0 ? nwtb0 : nwtb1); /* Width of filter table. */
            dm_cand_vec_t cdvmap = dm_cand_vec_map_to_finer(&cdv, na, nb, nwtb);
            /* Show the mapped candidates: */
            int ic;
            for (ic = 0; ic < cdvmap.nel; ic++)
              { fprintf(stderr, "  map[%03d]: ", ic);
                dm_cand_write(stderr, &(cdvmap.el[ic]), 6, 8, 6);
                fprintf(stderr, "\n");
              }
            if (levName != NULL)
              { /* Write and plot mapped candidates: */;
                dm_cand_vec_write_named(&cdvmap, levName, "-cd-map");
                dm_cand_vec_image_write_named(&cdvmap, na, circa, nb, circb, levName, "-cd-map");
              }
            cdv = cdvmap;
   
            /* Refine, sort, and prune as requested: */
            if (refine)
              { int nprune = (maxMatches + scale - 1)/scale;
                fprintf(stderr, "refining %d candidates, pruning to %d ...\n", cdvmap.nel, nprune);
                dm_cand_vec_t cdvref = dm_refine_cand_vec
                  ( &cdvmap, ap, bp, 
                    delta, kappa, shrink,
                    step_score, &tb, 
                    nprune, frac
                  );
                for (ic = 0; ic < cdvref.nel; ic++)
                  { fprintf(stderr, "  ref[%03d]: ", ic);
                    dm_cand_write(stderr, &(cdvref.el[ic]), 6, 8, 6);
                    fprintf(stderr, "\n");
                  }
                cdv = cdvref;
              }
          }
        
        /* Display the current set of candidates: */
        if (levName != NULL)
          { /* Write and plot refined candidates: */;
            dm_cand_vec_write_named(&cdv, levName, "-cd-cur");
            dm_cand_vec_image_write_named(&cdv, na, circa, nb, circb, levName, "-cd-cur");
          }
        
        fprintf(stderr, "= = = = = = = = = = = = = = = = = = = = = = = = = = = = = =\n");
        fprintf(stderr, "\n");
        free(levName);
      }
    fprintf(stderr, "mapped and refined %d candidates at finest level\n", cdv.nel);
    return cdv;
  }
