#ifndef dm_multi_H
#define dm_multi_H

/* Multiscale DNA matching */
/* Last edited on 2007-01-04 01:21:26 by stolfi */ 

#define dm_multi_H_COPYRIGHT \
  "Copyright  2005  by the State University of Campinas (UNICAMP)" \
  " and the Federal Fluminense University (UFF)"

#include <dm_basic.h>
#include <dm_seq.h>
#include <dm_rung.h>
#include <dm_cand.h>

void dm_multi_seq_filter
  ( dm_seq_t *s, 
    int maxLevel, 
    double_vec_t *wtb0, 
    char *wname0,
    double_vec_t *wtb1, 
    char *wname1,
    dm_seq_t sr[]
  );
  /* Places into {sr[0..maxLevel]} various filtered versions of the
    sequence {s}. Namely {sr[0]} is a copy of {s} (sharing the sample
    vector with {s}), and {sr[r]} is the filtered and down-sampled
    version of {sr[r-1]}, for {r} in {1..maxLevel}.  
    
    If a sequence {sr[k]} has {n} elements, then sequence {sr[k+1]}
    will have at most {(n-w+1)/2} elements, where {w} is the number of
    elements in the filter table. Therefore, the sequence {sr[k]} may
    be empty for {k} beyond a certain value.
    
    The table {wtb0} is used to go from {sr[0]} to {sr[1]}, and {wtb1}
    is used for the other scales; while the comment string {wname0} or {wname1},
    respectively, is to the {cmt} field of each new sequence. */

void dm_multi_seq_free(dm_seq_t sr[], int maxLevel);
  /* Reclaims the sample vectors of {sr[1..maxLelve]} (but not of {sr[0]}). */
    
void dm_multi_rung_vec_filter
  ( dm_rung_vec_t *gv, 
    int nx,
    bool_t circx,
    int ny,
    bool_t circy,
    int maxLevel, 
    int nw0, 
    int nw1, 
    dm_rung_vec_t gvr[]
  );
  /* Places into {gvr[0..maxLevel]} various filtered versions of the
    rung list {gv}. Namely, {gvr[0]} is a copy of {gv} (sharing the
    area {*(gv->el)}), and {gvr[r]} is the down-sampled version of
    {gvr[r-1]}, for {r} in {1..maxLevel}.
    
    Assumes that the unfiltered sequences connected by the rungs have
    {nx} and {ny} elements, respectively; that the filter table used
    for level 0 to level 1 has {nw0} entries; and the filter table
    used in the following stages has {nw1} entries. */

void dm_multi_rung_vec_free(dm_rung_vec_t gvr[], int maxLevel);
  /* Reclaims the sample vectors of {gvr[1..maxLelve]} (but not of {gvr[0]}). */
    
dm_cand_vec_t dm_multi_find_matches
  ( dm_cand_vec_t *cdvini,
    dm_seq_t a[], 
    dm_seq_t b[],
    int maxMatches,
    int maxLevel,
    int minBases,
    int nwtb0,
    int nwtb1,
    double frac,
    bool_t refine,
    int delta,
    int kappa,
    bool_t shrink,
    dm_step_score_proc_t *step_score,
    char *name
  );
  /* Finds good matches between two sequences {a[0]} and {b[0]} by
    multiscale matching up to recursion level {maxLevel}. Assumes that
    {a[r]} and {b[r]} are the sequences {a[0]} and {b[0]} filtered to
    scale {r}, for each {r} in {0..maxLevel}. Returns the best
    {maxMatches} result among those that have {minBases} or more
    bases.
    
    The client must provide the initial candidate list {cdvini}
    for sequences {a[maxLevel]} and {b[maxLevel]}. See
    {dm_cand_get_best_perfect}. 
    
    If the {refine} flag is true, at each scale {r} each candidate is
    re-optimized and its score is re-computed. Then the candidate list
    is trimmed to have at most {maxMatches*2^r} candidates, after
    discarding those that correspond to less than {minBases} bases in
    the finest scale.
    
    If {refine} is TRUE and {frac >= 0}, the procedure also discards
    any refined candidate {ca} that is mostly contained in some other
    candidate {cb} with better or equal score; where `mostly
    contained' means that the rungs of {ca} that are present in {cb}
    are at least {frac} times the number of rungs of {ca}.
    
    The parameters {nwtb0} and {nwtb1} should be the number of
    elements in the weight tables used for filtering: {nwtb0} from
    level 0 to level 1, {nwtb1} for all the other levels.
    
    The paramters {delta,kappa,shrink} are explained under {dm_cand_refine}.
    
    If {name} is not NULL, outputs several diagnostic files for 
    each level {NN}, from "00" to "{maxLevel-1}":
    
      "{name}-{NN}-eq.pgm"          Seq comparison image.
      "{name}-{NN}-cd-map.pgm"      Pairing image for mapped (unrefined) cands.
      "{name}-{NN}-cd-ref.pgm"      Pairing image for refined cands.
      "{name}-{NN}-mt-{icand}.pgm"  Dyn programming matrix for mapped cand {icand}.
    
    */

#endif
