#define PROG_NAME "dm_test_seqio"
#define PROG_DESC "test of DNA signal reading/writing routines"
#define PROG_VERS "1.0"

/* Last edited on 2007-01-04 00:12:16 by stolfi */

#define dm_test_seqio_C_COPYRIGHT \
  "Copyright  2006  by the State University of Campinas (UNICAMP)"

#define PROG_HELP \
  PROG_NAME " \\\n" \
  "  [ -circular ] {SEQ_FILE} \\\n" \
  "  {OUT_NAME}"

#define PROG_INFO \
  "NAME\n" \
  "  " PROG_NAME " - " PROG_DESC "\n" \
  "\n" \
  "SYNOPSIS\n" \
  "  " PROG_HELP "\n" \
  "\n" \
  "DESCRIPTION\n" \
  "  This program reads a" \
  " DNA/RNA sequence from {SEQ_FILE}, converts it to" \
  " numeric format, and outputs the numeric version of that" \
  " sequence.  The program then reads" \
  " that file and checks whether the data is preserved.  The" \
  " input file should be in \".seq\" format" \
  " (see the INPUT FILE FORMAT section below).\n" \
  "\n" \
  "INPUT FILE FORMAT\n" \
  dm_nucleic_file_format_INFO "\n" \
  "\n" \
  "OUTPUT FILES\n" \
  "  All output files will have names starting with {OUT_NAME}.\n" \
  "\n" \
  "OPTIONS\n" \
  "  -circular\n" \
  "    If present, specifies that the" \
  " sequence in {SEQ_FILE} is circular (like a bacterial" \
  " chromosome or plasmid).\n" \
  "\n" \
  argparser_help_info_HELP_INFO "\n" \
  "SEE ALSO\n" \
  "  dm_match(1)\n" \
  "\n" \
  "AUTHOR\n" \
  "  This program was created on 16/dec/2006 by J. Stolfi.\n" \
  "WARRANTY\n" \
  argparser_help_info_NO_WARRANTY "\n" \
  "\n" \
  "RIGHTS\n" \
  "  " dm_test_seqio_C_COPYRIGHT ".\n" \
  "\n" \
  argparser_help_info_STANDARD_RIGHTS  

#include <dm_basic.h>
#include <dm_seq.h>
#include <dm_nucleic.h>

#include <affirm.h>
#include <argparser.h>

#include <math.h>
#include <assert.h>
#include <string.h>
#include <stdlib.h>

typedef struct dm_options_t 
  { char *seqName;     /* Name of file with DNA/RNA sequence (minus ".bas" ext). */
    bool_t circular;   /* TRUE iff sequence is circular. */
    char *outName;     /* Output file name prefix (minus extensions). */
  } dm_options_t;
  
int main(int argc, char**argv);

dm_options_t *dm_get_options(int argc, char**argv);
  /* Parses the command line options, packs 
    them into a {dm_options_t} record. */

int main(int argc, char**argv)
  { 
    dm_options_t *o = dm_get_options(argc, argv);

    fprintf(stderr, "reading input DNA/RNA sequence ...\n");
    dm_seq_t s = dm_seq_read_from_nucleic_file(0, "s", o->seqName, o->circular);

    /* Create output filename: */
    fprintf(stderr, "writing numeric DNA/RNA sequence...\n");
    FILE *wr = dm_open_write(o->outName, "", ".egs", TRUE);
    dm_seq_write(wr, &s);
    fclose(wr);
    
    fprintf(stderr, "reading numeric DNA/RNA sequence back in...\n");
    FILE *rd = dm_open_read(o->outName, "", ".egs", TRUE);
    dm_seq_t t = dm_seq_read(rd);
    fclose(rd);
    
    fprintf(stderr, "checking...\n");
    assert(t.id == dm_seq_id_none);
    assert(t.name == NULL);
    assert(strcmp(s.cmt, t.cmt) == 0);
    assert(s.circ == t.circ);
    assert(s.dv.nel == t.dv.nel);
    int k;
    for (k = 0; k < dm_CHANNELS; k++)
      { assert(s.sfac.f[k] == t.sfac.f[k]); 
        int i;
        for (i = 0; i < s.dv.nel; i++)
          { dm_sample_t ssmp = dm_seq_get_sample(&s, i, k); 
            dm_sample_t tsmp = dm_seq_get_sample(&t, i, k);
            assert(ssmp == tsmp);
          }
      }
    
    /* ...and we are done: */
    fprintf(stderr, "done.\n");
    return 0;
  }

dm_options_t* dm_get_options(int argc, char**argv)
  { dm_options_t *o = (dm_options_t *)notnull(malloc(sizeof(dm_options_t)), "no mem");
    
    argparser_t *pp = argparser_new(stderr, argc, argv);
    argparser_set_help(pp, PROG_HELP);
    argparser_set_info(pp, PROG_INFO);
    argparser_process_help_info_options(pp);
    
    o->circular = argparser_keyword_present(pp, "-circular");

    argparser_skip_parsed(pp);
    
    o->seqName = argparser_get_next(pp);
    
    o->outName = argparser_get_next(pp);

    argparser_finish(pp);
    
    return o;
  }
