#ifndef dm_test_tools_H
#define dm_test_tools_H

/* Miscellaneous tools for test programs. */
/* Last edited on 2007-01-04 02:40:19 by stolfi */

#define dm_test_tools_H_COPYRIGHT \
  "Copyright  2006  by the State University of Campinas (UNICAMP)"

#include <dm_basic.h>
#include <dm_seq.h>
#include <dm_pairing.h>
#include <dm_cand.h>
#include <dm_dyn.h>

#include <vec.h>

dm_seq_t dm_test_tools_make_seq
  ( dm_seq_id_t id, 
    char *tag, 
    char *borg, 
    double mutProb, 
    double delProb, 
    bool_t circ,
    char *outName
    );
  /* Generates a test sequence by mutating the nuceotide string {borg}
    with mutation probability {mutProb} and deletion/insertion
    probability {delProb}. The sequence will have numeric identifier
    {id}, name {side}, and an appropriate {cmt} string. The sequence is
    circular iff {circ} is TRUE. Also writes the sequence to file
    "{outName}-{tag}.bas". */

void dm_test_tools_make_seq_pair
  ( char *borg, 
    double mutProb, 
    double delProb, 
    bool_t circ,
    dm_seq_id_t xid, 
    char *xtag,
    dm_seq_t *xP,
    dm_seq_id_t yid, 
    char *ytag,
    dm_seq_t *yP,
    dm_rung_vec_t *gv,
    char *outName
  );
  /* Generates two test sequences {*xP,*yP} by mutating the nuceotide string {borg}
    with mutation probability {mutProb} and deletion/insertion
    probability {delProb}. The sequences will have numeric identifier
    {xid,yid}, name {xtag,ytag}, and appropriate {cmt} strings. The sequences 
    are circular iff {circ} is TRUE.  
    
    The procedure also returns in {*gv} the list of `true' rungs between
    between the two sequences, i.e. the list of pairs {ix,iy} such that
    {xP[ix]} and {yP[iy]} were both copied from the same letter of 
    {borg}, without mutation.  Beware that {pr} will be monotonic but
    not unitary in general.
    
    The procedure also writes the sequences to files
    "{outName}-{xtag}.bas" and "{outName}-{ytag}.bas". */

void dm_test_tools_write_and_plot_multi_seq(dm_seq_t r[], int maxLevel, char *outName);
  /* Writes out the sequences {r[0..maxLevel]} to files called "{outName}-{NN}.egs"
    where {NN} is the level.  Also writes their graphs as Encapsulated Postscript
    files "{outName}-{NN}.eps". */

dm_cand_t dm_test_tools_get_optimum_pairing
  ( dm_seq_t *xp, 
    dm_seq_t *yp, 
    int delta, 
    dm_step_score_proc_t *step_score, 
    dm_dyn_tableau_t *tb
  );
  /* Finds an optimum pairing between the sequences {ap} and {bp}.
    The optimum pairing is sought within 
    a band of half-width {delta} around the Bresenham pairing
    that covers both sequences once. */

#endif

